#!/usr/bin/python

# This script was tested with IUPred version 1.0

from subprocess import Popen, PIPE

from sys import argv, stdin, stdout

from re import findall


def parse_FASTA(fasta_file):
    
    if isinstance(fasta_file, str):
        fasta = fasta_file.split("\n")
    else:
        fasta = fasta_file
    
    # Variables for holding transitory informations
    current_name = ""
    current_sequence = ""
    
    # For each line in the file:
    for line in fasta:
        
		# Ignore it if it is a comment
        if not line.startswith("#"):
            
            # If it is the start of a new fasta entry:
            if line.startswith(">"):
                
                # If it is not the first entry of the file:
                if current_name:
                    
                    # Yield the parsed fasta entry
                    yield (current_name, current_sequence)
                    
                    # Reset the sequence
                    current_sequence = ""
                
                # Change the "current name" to the new sequence name
                current_name = line.rstrip()[1:]
            
            # If it is a sequence line, add the new sequence chunk to the current sequence
            else:
                current_sequence += line.rstrip()
    
    # Once all lines have been read, set the last dictionary item
    yield (current_name, current_sequence)


def write_FASTA(seq_iter, connection = None):
    
    # String for holding the whole FASTA file
    if not connection:
        fasta_string = ""
    
    # For each item in the sequence iterator:
    for name, seq in seq_iter:
        
        # Split the sequence in 60 characters lines and join them with newlines
        sequence_lines = "\n".join(findall(".{1,60}", seq)) + "\n"
        
        # Complete the FASTA entry by adding the name line at the top
        fasta_entry = ">" + name + "\n" + sequence_lines
        
        # Add the FASTA entry to the file or string
        if connection:
            connection.write(fasta_entry)
        else:
            fasta_string += fasta_entry
    
    # Once the writing is completed, return the FASTA string if need be
    if not connection:
        return fasta_string

def parse_iupred(file_iter, pred_type):
    
    """Parse an IUPred output file (iterator of lines) and return a list
    of per-residue disorder probabilities. IUPred must have been used
    in 'short' mode or 'long' mode, but not "glob" mode. pred_type
    should be either 'short' or 'long', accordingly."""
    
    if pred_type in ["long", "short"]:
        
        return [
            float(line.split()[2])
            for line in file_iter
            if line and not line.startswith("#")
        ]
        
    else:
        raise ValueError("Accepted pred_type values are 'long' and 'short'.")


def run_iupred(fasta_file, pred_type):
    
    """Return a generator that yields pairs of the form (name, probs), 
    where name is the identifier of a sequence in the fasta_file, and
    probs is a list of per-residue disorder probabilities, as computed
    by the function parse_iupred. pred_type should be either 'short' or
    'long'."""
    
    for seq_name, seq in parse_FASTA(fasta_file):
        
        iupred_pipe = Popen(
            "iupred /dev/stdin " + pred_type,
            stdin = PIPE,
            stdout = PIPE,
            stderr = PIPE,
            shell=True
        )
        
        iupred_input = write_FASTA([(seq_name, seq)])
        
        iupred_output, iupred_error = iupred_pipe.communicate(iupred_input)
        
        if iupred_error:
            raise RuntimeError("IUPred error message: " + iupred_error)
        
        iupred_probs = parse_iupred(
            iupred_output.split("\n"),
            pred_type
        )
        
        yield (seq_name, iupred_probs)



# Write a tab-separated file to standard output. First column contains
# sequence names, other columns contain per-residue disorder
# probabilities. The number of elements in each line is one plus the
# length of the peptide sequence.

if __name__ == "__main__":
    
    fasta_name, prediction_type = argv[1:]
    
    if fasta_name == "-":
        
        output_iter = run_iupred(stdin, prediction_type)
        
        for seq_name, scores in output_iter:
            
            scores_as_strings = map(str, scores)
            
            stdout.write("\t".join([seq_name] + scores_as_strings) + "\n")
        
    else:
        
        with open(fasta_name) as fasta:
            
            output_iter = run_iupred(fasta, prediction_type)
            
            for seq_name, scores in output_iter:
                
                scores_as_strings = map(str, scores)
                
                stdout.write("\t".join([seq_name] + scores_as_strings) + "\n")
            


